@@ -30,11 +30,9 @@
 #include <QFontMetrics>
 #include <cmath>
 
-#include <KDirWatch>
+#include <KConfigWatcher>
 #include <KIconLoader>
 
-QString plasmarc() { return QStringLiteral("plasmarc"); }
-QString groupName() { return QStringLiteral("Units"); }
 const int defaultLongDuration = 120;
 
 
@@ -72,21 +70,21 @@ Units::Units(QObject *parent)
     }
 
     m_iconSizes = new QQmlPropertyMap(this);
+    m_iconSizeHints = new QQmlPropertyMap(this);
     updateDevicePixelRatio(); // also updates icon sizes
     updateSpacing(); // updates gridUnit and *Spacing properties
 
     connect(KIconLoader::global(), &KIconLoader::iconLoaderSettingsChanged, this, &Units::iconLoaderSettingsChanged);
-    QObject::connect(s_sharedAppFilter, SIGNAL(fontChanged()), this, SLOT(updateSpacing()));
+    QObject::connect(s_sharedAppFilter, &SharedAppFilter::fontChanged, this, &Units::updateSpacing);
 
-    const QString configFile = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + QLatin1Char('/') + plasmarc();
-    KDirWatch::self()->addFile(configFile);
-
-    // Catch both, direct changes to the config file ...
-    connect(KDirWatch::self(), &KDirWatch::dirty, this, &Units::settingsFileChanged);
-    // ... but also remove/recreate cycles, like KConfig does it
-    connect(KDirWatch::self(), &KDirWatch::created, this, &Units::settingsFileChanged);
-    // read configuration
-    updatePlasmaRCSettings();
+    m_animationSpeedWatcher = KConfigWatcher::create(KSharedConfig::openConfig());
+    connect(m_animationSpeedWatcher.data(), &KConfigWatcher::configChanged, this,
+        [this](const KConfigGroup &group, const QByteArrayList &names) {
+            if (group.name() == QLatin1String("KDE") && names.contains(QByteArrayLiteral("AnimationDurationFactor"))) {
+                updateAnimationSpeed();
+            }
+    });
+    updateAnimationSpeed();
 }
 
 Units::~Units()
@@ -100,21 +98,20 @@ Units &Units::instance()
     return units;
 }
 
-void Units::settingsFileChanged(const QString &file)
+void Units::updateAnimationSpeed()
 {
-    if (file.endsWith(plasmarc())) {
-        KSharedConfigPtr cfg = KSharedConfig::openConfig(plasmarc());
-        cfg->reparseConfiguration();
-        updatePlasmaRCSettings();
-    }
-}
+    KConfigGroup generalCfg = KConfigGroup(KSharedConfig::openConfig(), QStringLiteral("KDE"));
+    const qreal animationSpeedModifier = qMax(0.0, generalCfg.readEntry("AnimationDurationFactor", 1.0));
+
+    // Read the old longDuration value for compatibility
+    KConfigGroup cfg = KConfigGroup(KSharedConfig::openConfig(QStringLiteral("plasmarc")), QStringLiteral("Units"));
+    int longDuration = cfg.readEntry("longDuration", defaultLongDuration);
+
+    longDuration = qRound(longDuration * animationSpeedModifier);
 
-void Units::updatePlasmaRCSettings()
-{
-    KConfigGroup cfg = KConfigGroup(KSharedConfig::openConfig(plasmarc()), groupName());
     // Animators with a duration of 0 do not fire reliably
     // see Bug 357532 and QTBUG-39766
-    const int longDuration = qMax(1, cfg.readEntry("longDuration", defaultLongDuration));
+    longDuration = qMax(1, longDuration);
 
     if (longDuration != m_longDuration) {
         m_longDuration = longDuration;
@@ -135,7 +132,11 @@ void Units::iconLoaderSettingsChanged()
     m_iconSizes->insert(QStringLiteral("huge"), devicePixelIconSize(KIconLoader::SizeHuge));
     m_iconSizes->insert(QStringLiteral("enormous"), devicePixelIconSize(KIconLoader::SizeEnormous));
 
+    m_iconSizeHints->insert(QStringLiteral("panel"), devicePixelIconSize(KIconLoader::global()->currentSize(KIconLoader::Panel)));
+    m_iconSizeHints->insert(QStringLiteral("desktop"), devicePixelIconSize(KIconLoader::global()->currentSize(KIconLoader::Desktop)));
+
     emit iconSizesChanged();
+    emit iconSizeHintsChanged();
 }
 
 QQmlPropertyMap *Units::iconSizes() const
@@ -143,6 +144,11 @@ QQmlPropertyMap *Units::iconSizes() const
     return m_iconSizes;
 }
 
+QQmlPropertyMap *Units::iconSizeHints() const
+{
+    return m_iconSizeHints;
+}
+
 int Units::roundToIconSize(int size)
 {
     /*Do *not* use devicePixelIconSize here, we want to use the sizes of the pixmaps of the smallest icons on the disk. And those are unaffected by dpi*/
