@@ -27,13 +27,18 @@ namespace Plasma
 class Theme;
 }
 
+//TODO: should be possible to remove without significant breakages as is just an image provider
+/**
+ * image provider for textures used for applications
+ * @deprecated don't use it
+ */
 class AppBackgroundProvider : public QQuickImageProvider
 {
 
 public:
     AppBackgroundProvider();
-    ~AppBackgroundProvider();
-    QImage requestImage(const QString &id, QSize *size, const QSize &requestedSize) Q_DECL_OVERRIDE;
+    ~AppBackgroundProvider() override;
+    QImage requestImage(const QString &id, QSize *size, const QSize &requestedSize) override;
 private:
     Plasma::Theme *m_theme;
 };
