@@ -22,8 +22,11 @@
 
 #include <QDebug>
 #include <QAbstractItemModel>
+#include <QRandomGenerator>
 
 #include "plasma.h"
+#include "debug_p.h"
+
 
 namespace Plasma
 {
@@ -106,7 +109,7 @@ bool DataContainer::visualizationIsConnected(QObject *visualization) const
 void DataContainer::connectVisualization(QObject *visualization, uint pollingInterval,
         Plasma::Types::IntervalAlignment alignment)
 {
-    //qDebug() << "connecting visualization" <<this<< visualization << "at interval of"
+    //qCDebug(LOG_PLASMA) << "connecting visualization" <<this<< visualization << "at interval of"
     //         << pollingInterval << "to" << objectName();
     QMap<QObject *, SignalRelay *>::iterator objIt = d->relayObjects.find(visualization);
     bool connected = objIt != d->relayObjects.end();
@@ -116,45 +119,53 @@ void DataContainer::connectVisualization(QObject *visualization, uint pollingInt
         SignalRelay *relay = objIt.value();
         if (relay) {
             // connected to a relay
-            //qDebug() << "     already connected, but to a relay";
+            //qCDebug(LOG_PLASMA) << "     already connected, but to a relay";
             if (relay->m_interval == pollingInterval && relay->m_align == alignment) {
-                //qDebug() << "    already connected to a relay of the same interval of"
+                //qCDebug(LOG_PLASMA) << "    already connected to a relay of the same interval of"
                 //          << pollingInterval << ", nothing to do";
                 return;
             }
 
             if (relay->receiverCount() == 1) {
-                //qDebug() << "    removing relay, as it is now unused";
+                //qCDebug(LOG_PLASMA) << "    removing relay, as it is now unused";
                 d->relays.remove(relay->m_interval);
                 delete relay;
             } else {
-                disconnect(relay, SIGNAL(dataUpdated(QString,Plasma::DataEngine::Data)),
-                           visualization, SLOT(dataUpdated(QString,Plasma::DataEngine::Data)));
+                if (visualization->metaObject()->indexOfSlot("dataUpdated(QString,Plasma::DataEngine::Data)") >= 0) {
+                    disconnect(relay, SIGNAL(dataUpdated(QString,Plasma::DataEngine::Data)),
+                               visualization, SLOT(dataUpdated(QString,Plasma::DataEngine::Data)));
+                }
                 //modelChanged is always emitted by the dataSource since there is no polling there
-                disconnect(this, SIGNAL(modelChanged(QString,QAbstractItemModel*)),
-                           visualization, SLOT(modelChanged(QString,QAbstractItemModel*)));
+                if (visualization->metaObject()->indexOfSlot("modelChanged(QString,QAbstractItemModel*)") >= 0) {
+                        disconnect(this, SIGNAL(modelChanged(QString,QAbstractItemModel*)),
+                                visualization, SLOT(modelChanged(QString,QAbstractItemModel*)));
+                }
                 //relay->isUnused();
             }
         } else if (pollingInterval < 1) {
             // the visualization was connected already, but not to a relay
             // and it still doesn't want to connect to a relay, so we have
             // nothing to do!
-            //qDebug() << "     already connected, nothing to do";
+            //qCDebug(LOG_PLASMA) << "     already connected, nothing to do";
             return;
         } else {
-            disconnect(this, SIGNAL(dataUpdated(QString,Plasma::DataEngine::Data)),
-                       visualization, SLOT(dataUpdated(QString,Plasma::DataEngine::Data)));
-            disconnect(this, SIGNAL(modelChanged(QString,QAbstractItemModel*)),
-                       visualization, SLOT(modelChanged(QString,QAbstractItemModel*)));
+            if (visualization->metaObject()->indexOfSlot("dataUpdated(QString,Plasma::DataEngine::Data)") >= 0) {
+                disconnect(this, SIGNAL(dataUpdated(QString,Plasma::DataEngine::Data)),
+                           visualization, SLOT(dataUpdated(QString,Plasma::DataEngine::Data)));
+            }
+            if (visualization->metaObject()->indexOfSlot("modelChanged(QString,QAbstractItemModel*)") >= 0) {
+                disconnect(this, SIGNAL(modelChanged(QString,QAbstractItemModel*)),
+                    visualization, SLOT(modelChanged(QString,QAbstractItemModel*)));
+            }
         }
     } else {
-        connect(visualization, SIGNAL(destroyed(QObject*)),
-                this, SLOT(disconnectVisualization(QObject*))); //, Qt::QueuedConnection);
+        connect(visualization, &QObject::destroyed,
+                this, &DataContainer::disconnectVisualization); //, Qt::QueuedConnection);
     }
 
     if (pollingInterval < 1) {
-        //qDebug() << "    connecting directly";
-        d->relayObjects[visualization] = 0;
+        //qCDebug(LOG_PLASMA) << "    connecting directly";
+        d->relayObjects[visualization] = nullptr;
         if (visualization->metaObject()->indexOfSlot("dataUpdated(QString,Plasma::DataEngine::Data)") >= 0) {
             connect(this, SIGNAL(dataUpdated(QString,Plasma::DataEngine::Data)),
                     visualization, SLOT(dataUpdated(QString,Plasma::DataEngine::Data)));
@@ -164,28 +175,30 @@ void DataContainer::connectVisualization(QObject *visualization, uint pollingInt
                     visualization, SLOT(modelChanged(QString,QAbstractItemModel*)));
         }
     } else {
-        //qDebug() << "    connecting to a relay";
-        // we only want to do an imediate update if this is not the first object to connect to us
+        //qCDebug(LOG_PLASMA) << "    connecting to a relay";
+        // we only want to do an immediate update if this is not the first object to connect to us
         // if it is the first visualization, then the source will already have been populated
         // engine's sourceRequested method
         bool immediateUpdate = connected || d->relayObjects.count() > 1;
         SignalRelay *relay = d->signalRelay(this, visualization, pollingInterval,
                                             alignment, immediateUpdate);
-        connect(relay, SIGNAL(dataUpdated(QString,Plasma::DataEngine::Data)),
-                visualization, SLOT(dataUpdated(QString,Plasma::DataEngine::Data)));
+        if (visualization->metaObject()->indexOfSlot("dataUpdated(QString,Plasma::DataEngine::Data)") >= 0) {
+            connect(relay, SIGNAL(dataUpdated(QString,Plasma::DataEngine::Data)),
+                    visualization, SLOT(dataUpdated(QString,Plasma::DataEngine::Data)));
+        }
         //modelChanged is always emitted by the dataSource since there is no polling there
-        connect(this, SIGNAL(modelChanged(QString,QAbstractItemModel*)),
+        if (visualization->metaObject()->indexOfSlot("modelChanged(QString,QAbstractItemModel*)") >= 0) {
+            connect(this, SIGNAL(modelChanged(QString,QAbstractItemModel*)),
                 visualization, SLOT(modelChanged(QString,QAbstractItemModel*)));
+        }
     }
 }
 
 void DataContainer::setStorageEnabled(bool store)
 {
-    QTime time = QTime::currentTime();
-    qsrand((uint)time.msec());
     d->enableStorage = store;
     if (store) {
-        QTimer::singleShot(qrand() % (2000 + 1), this, SLOT(retrieve()));
+        QTimer::singleShot(QRandomGenerator::global()->bounded(2000 + 1), this, SLOT(retrieve()));
     }
 }
 
@@ -207,11 +220,11 @@ void DataContainer::setNeedsToBeStored(bool store)
 DataEngine *DataContainer::getDataEngine()
 {
     QObject *o = this;
-    DataEngine *de = NULL;
-    while (de == NULL) {
+    DataEngine *de = nullptr;
+    while (de == nullptr) {
         o = dynamic_cast<QObject *>(o->parent());
-        if (o == NULL) {
-            return NULL;
+        if (o == nullptr) {
+            return nullptr;
         }
         de = dynamic_cast<DataEngine *>(o);
     }
@@ -235,8 +248,8 @@ void DataContainerPrivate::store()
         storage = new Storage(q);
     }
 
-    QVariantMap op = storage->operationDescription("save");
-    op["group"] = q->objectName();
+    QVariantMap op = storage->operationDescription(QStringLiteral("save"));
+    op[QStringLiteral("group")] = q->objectName();
     StorageJob *job = static_cast<StorageJob *>(storage->startOperationCall(op));
     job->setData(data);
     storageCount++;
@@ -248,14 +261,14 @@ void DataContainerPrivate::storeJobFinished(KJob *)
     --storageCount;
     if (storageCount < 1) {
         storage->deleteLater();
-        storage = 0;
+        storage = nullptr;
     }
 }
 
 void DataContainerPrivate::retrieve()
 {
     DataEngine *de = q->getDataEngine();
-    if (de == NULL) {
+    if (de == nullptr) {
         return;
     }
 
@@ -263,8 +276,8 @@ void DataContainerPrivate::retrieve()
         storage = new Storage(q);
     }
 
-    QVariantMap retrieveGroup = storage->operationDescription("retrieve");
-    retrieveGroup["group"] = q->objectName();
+    QVariantMap retrieveGroup = storage->operationDescription(QStringLiteral("retrieve"));
+    retrieveGroup[QStringLiteral("group")] = q->objectName();
     ServiceJob *retrieveJob = storage->startOperationCall(retrieveGroup);
     QObject::connect(retrieveJob, SIGNAL(result(KJob*)), q,
                      SLOT(populateFromStoredData(KJob*)));
@@ -289,24 +302,28 @@ void DataContainerPrivate::populateFromStoredData(KJob *job)
         q->forceImmediateUpdate();
     }
 
-    QVariantMap expireGroup = storage->operationDescription("expire");
+    QVariantMap expireGroup = storage->operationDescription(QStringLiteral("expire"));
     //expire things older than 4 days
-    expireGroup["age"] = 345600;
+    expireGroup[QStringLiteral("age")] = 345600;
     storage->startOperationCall(expireGroup);
 }
 
 void DataContainer::disconnectVisualization(QObject *visualization)
 {
     QMap<QObject *, SignalRelay *>::iterator objIt = d->relayObjects.find(visualization);
-    disconnect(visualization, SIGNAL(destroyed(QObject*)),
-               this, SLOT(disconnectVisualization(QObject*))); //, Qt::QueuedConnection);
+    disconnect(visualization, &QObject::destroyed,
+               this, &DataContainer::disconnectVisualization); //, Qt::QueuedConnection);
 
     if (objIt == d->relayObjects.end() || !objIt.value()) {
         // it is connected directly to the DataContainer itself
-        disconnect(this, SIGNAL(dataUpdated(QString,Plasma::DataEngine::Data)),
-                   visualization, SLOT(dataUpdated(QString,Plasma::DataEngine::Data)));
-        disconnect(this, SIGNAL(modelChanged(QString,QAbstractItemModel*)),
+        if (visualization->metaObject()->indexOfSlot("dataUpdated(QString,Plasma::DataEngine::Data)") >= 0) {
+            disconnect(this, SIGNAL(dataUpdated(QString,Plasma::DataEngine::Data)),
+                       visualization, SLOT(dataUpdated(QString,Plasma::DataEngine::Data)));
+        }
+        if (visualization->metaObject()->indexOfSlot("modelChanged(QString,QAbstractItemModel*)") >= 0) {
+            disconnect(this, SIGNAL(modelChanged(QString,QAbstractItemModel*)),
                    visualization, SLOT(modelChanged(QString,QAbstractItemModel*)));
+        }
     } else {
         SignalRelay *relay = objIt.value();
 
@@ -314,11 +331,15 @@ void DataContainer::disconnectVisualization(QObject *visualization)
             d->relays.remove(relay->m_interval);
             delete relay;
         } else {
-            disconnect(relay, SIGNAL(dataUpdated(QString,Plasma::DataEngine::Data)),
-                       visualization, SLOT(dataUpdated(QString,Plasma::DataEngine::Data)));
+            if (visualization->metaObject()->indexOfSlot("dataUpdated(QString,Plasma::DataEngine::Data)") >= 0) {
+                disconnect(relay, SIGNAL(dataUpdated(QString,Plasma::DataEngine::Data)),
+                           visualization, SLOT(dataUpdated(QString,Plasma::DataEngine::Data)));
+            }
             //modelChanged is always emitted by the dataSource since there is no polling there
-            disconnect(this, SIGNAL(modelChanged(QString,QAbstractItemModel*)),
-                       visualization, SLOT(modelChanged(QString,QAbstractItemModel*)));
+            if (visualization->metaObject()->indexOfSlot("modelChanged(QString,QAbstractItemModel*)") >= 0) {
+                    disconnect(this, SIGNAL(modelChanged(QString,QAbstractItemModel*)),
+                        visualization, SLOT(modelChanged(QString,QAbstractItemModel*)));
+            }
         }
     }
 
@@ -328,7 +349,7 @@ void DataContainer::disconnectVisualization(QObject *visualization)
 
 void DataContainer::checkForUpdate()
 {
-    //qDebug() << objectName() << d->dirty;
+    //qCDebug(LOG_PLASMA) << objectName() << d->dirty;
     if (d->dirty) {
         emit dataUpdated(objectName(), d->data);
 
@@ -380,12 +401,12 @@ void DataContainer::timerEvent(QTimerEvent *event)
     if (event->timerId() == d->checkUsageTimer.timerId()) {
         if (!isUsed()) {
             // DO NOT CALL ANYTHING AFTER THIS LINE AS IT MAY GET DELETED!
-            //qDebug() << objectName() << "is unused";
+            //qCDebug(LOG_PLASMA) << objectName() << "is unused";
 
             //NOTE: Notifying visualization of the model destruction before actual deletion avoids crashes in some edge cases
             if (d->model) {
                 d->model.clear();
-                emit modelChanged(objectName(), 0);
+                emit modelChanged(objectName(), nullptr);
             }
             emit becameUnused(objectName());
         }
