@@ -28,9 +28,9 @@ class ThemePackage : public KPackage::PackageStructure
 {
     Q_OBJECT
 public:
-    ThemePackage(QObject *parent = 0, const QVariantList &args = QVariantList()) : KPackage::PackageStructure(parent, args) {}
+    ThemePackage(QObject *parent = nullptr, const QVariantList &args = QVariantList()) : KPackage::PackageStructure(parent, args) {}
 
-    void initPackage(KPackage::Package *package) Q_DECL_OVERRIDE
+    void initPackage(KPackage::Package *package) override
     {
         // by default the packages have "contents/" as contentsPrefixPaths
         // but for the themes we don't want that, so unset it.
