@@ -106,13 +106,13 @@ public:
     Q_ENUM(ContainmentType)
 
     /**
-     * A descriptrive type for QActions, to help categorizing them when presented to the user
+     * A descriptive type for QActions, to help categorizing them when presented to the user
      */
     enum ActionType {
         AddAction = 0,   /**< The action will cause something new being created*/
         ConfigureAction = 100,   /**< The Action will make some kind of configuration ui to appear */
         ControlAction = 200,  /**< Generic control, similar to ConfigureAction TODO: better doc */
-        MiscAction = 300,  /**< A type of action that doesn't fit in the oher categories */
+        MiscAction = 300,  /**< A type of action that doesn't fit in the other categories */
         DestructiveAction = 400,  /**< A dangerous action, such as deletion of objects, plasmoids and files. They are intended to be shown separated from other actions */
         UserAction = DestructiveAction + 1000 /**< If new types are needed in a C++ implementation, define them as ids more than  UserAction*/
     };
@@ -122,7 +122,7 @@ public:
      * The Direction enumeration describes in which direction, relative to the
      * Applet (and its managing container), popup menus, expanders, balloons,
      * message boxes, arrows and other such visually associated widgets should
-     * appear in. This is usually the oposite of the Location.
+     * appear in. This is usually the opposite of the Location.
      **/
     enum Direction {
         Down = 0, /**< Display downards */
@@ -169,19 +169,19 @@ public:
     enum PopupPlacement {
         FloatingPopup = 0,            /**< Free floating, non attached popup */
         TopPosedLeftAlignedPopup,     /**< Popup positioned on the top, aligned
-                                     to the left of the wigdet */
+                                     to the left of the widget */
         TopPosedRightAlignedPopup,    /**< Popup positioned on the top, aligned
                                      to the right of the widget */
         LeftPosedTopAlignedPopup,     /**< Popup positioned on the left, aligned
-                                     to the top of the wigdet */
+                                     to the top of the widget */
         LeftPosedBottomAlignedPopup,  /**< Popup positioned on the left, aligned
                                      to the bottom of the widget */
         BottomPosedLeftAlignedPopup,  /**< Popup positioned on the bottom, aligned
-                                     to the left of the wigdet */
+                                     to the left of the widget */
         BottomPosedRightAlignedPopup, /**< Popup positioned on the bottom, aligned
                                      to the right of the widget */
         RightPosedTopAlignedPopup,    /**< Popup positioned on the right, aligned
-                                     to the top of the wigdet */
+                                     to the top of the widget */
         RightPosedBottomAlignedPopup  /**< Popup positioned on the right, aligned
                                      to the bottom of the widget */
     };
@@ -280,9 +280,13 @@ public:
         NoBackground = 0,         /**< Not drawing a background under the applet, the applet has its own implementation */
         StandardBackground = 1,   /**< The standard background from the theme is drawn */
         TranslucentBackground = 2, /**< An alternate version of the background is drawn, usually more translucent */
+        ShadowBackground = 4, /**< The applet won't have a svg background but a drop shadow of its content done via a shader */
+        ConfigurableBackground = 8, /** If the hint has this flag, the user is able to configure this background */
         DefaultBackground = StandardBackground /**< Default settings: both standard background */
     };
     Q_ENUM(BackgroundHints)
+    //TODO KF6: BackgroundHint and BackgroundHints
+    Q_DECLARE_FLAGS(BackgroundFlags, BackgroundHints)
 
 private:
     Types(QObject *parent = nullptr);
@@ -311,5 +315,5 @@ PLASMA_EXPORT Types::Direction locationToInverseDirection(Types::Location locati
 Q_DECLARE_OPERATORS_FOR_FLAGS(Plasma::Types::Constraints)
 Q_DECLARE_OPERATORS_FOR_FLAGS(Plasma::Types::Flip)
 Q_DECLARE_OPERATORS_FOR_FLAGS(Plasma::Types::ComponentTypes)
-
+Q_DECLARE_OPERATORS_FOR_FLAGS(Plasma::Types::BackgroundFlags)
 #endif // multiple inclusion guard
