@@ -84,18 +84,18 @@ ThemePrivate::ThemePrivate(QObject *parent)
     pixmapSaveTimer = new QTimer(this);
     pixmapSaveTimer->setSingleShot(true);
     pixmapSaveTimer->setInterval(600);
-    QObject::connect(pixmapSaveTimer, SIGNAL(timeout()), this, SLOT(scheduledCacheUpdate()));
+    QObject::connect(pixmapSaveTimer, &QTimer::timeout, this, &ThemePrivate::scheduledCacheUpdate);
 
     rectSaveTimer = new QTimer(this);
     rectSaveTimer->setSingleShot(true);
     //2 minutes
     rectSaveTimer->setInterval(2 * 60 * 1000);
-    QObject::connect(rectSaveTimer, SIGNAL(timeout()), this, SLOT(saveSvgElementsCache()));
+    QObject::connect(rectSaveTimer, &QTimer::timeout, this, &ThemePrivate::saveSvgElementsCache);
 
     updateNotificationTimer = new QTimer(this);
     updateNotificationTimer->setSingleShot(true);
     updateNotificationTimer->setInterval(100);
-    QObject::connect(updateNotificationTimer, SIGNAL(timeout()), this, SLOT(notifyOfChanged()));
+    QObject::connect(updateNotificationTimer, &QTimer::timeout, this, &ThemePrivate::notifyOfChanged);
 
     if (QPixmap::defaultDepth() > 8) {
 #if HAVE_X11
@@ -200,11 +200,11 @@ bool ThemePrivate::useCache()
 
                 // watch the metadata file for changes at runtime
                 KDirWatch::self()->addFile(themeMetadataPath);
-                QObject::connect(KDirWatch::self(), SIGNAL(created(QString)),
-                                 this, SLOT(settingsFileChanged(QString)),
+                QObject::connect(KDirWatch::self(), &KDirWatch::created,
+                                 this, &ThemePrivate::settingsFileChanged,
                                  Qt::UniqueConnection);
-                QObject::connect(KDirWatch::self(), SIGNAL(dirty(QString)),
-                                 this, SLOT(settingsFileChanged(QString)),
+                QObject::connect(KDirWatch::self(), &KDirWatch::dirty,
+                                 this, &ThemePrivate::settingsFileChanged,
                                  Qt::UniqueConnection);
 
                 if (!iconThemeMetadataPath.isEmpty()) {
@@ -303,7 +303,7 @@ void ThemePrivate::onAppExitCleanup()
 
 QString ThemePrivate::imagePath(const QString& theme, const QString& type, const QString& image)
 {
-    QString subdir = QLatin1Literal(PLASMA_RELATIVE_DATA_INSTALL_DIR "/desktoptheme/") % theme % type % image;
+    QString subdir = QLatin1String(PLASMA_RELATIVE_DATA_INSTALL_DIR "/desktoptheme/") % theme % type % image;
     return QStandardPaths::locate(QStandardPaths::GenericDataLocation, subdir);
 }
 
@@ -316,7 +316,7 @@ QString ThemePrivate::findInTheme(const QString &image, const QString &theme, bo
         }
     }
 
-    QString type;
+    QString type = QStringLiteral("/");
     if (!compositingActive) {
         type = QStringLiteral("/opaque/");
     } else if (backgroundContrastActive) {
@@ -400,6 +400,7 @@ void ThemePrivate::colorsChanged()
     buttonColorScheme = KColorScheme(QPalette::Active, KColorScheme::Button, colors);
     viewColorScheme = KColorScheme(QPalette::Active, KColorScheme::View, colors);
     selectionColorScheme = KColorScheme(QPalette::Active, KColorScheme::Selection, colors);
+    palette = KColorScheme::createApplicationPalette(colors);
     scheduleThemeChangeNotification(PixmapCache | SvgElementsCache);
     emit applicationPaletteChange();
 }
@@ -615,7 +616,7 @@ void ThemePrivate::saveSvgElementsCache()
         while (it.hasNext()) {
             it.next();
             KConfigGroup imageGroup(svgElementsCache, it.key());
-            imageGroup.writeEntry("invalidElements", it.value().toList()); //FIXME: add QSet support to KConfig
+            imageGroup.writeEntry("invalidElements", it.value().values()); //FIXME: add QSet support to KConfig
         }
 
         //Pretty drastic, but this is executed only very rarely
@@ -692,6 +693,8 @@ QColor ThemePrivate::color(Theme::ColorRole role, Theme::ColorGroup group) const
         return scheme->foreground(KColorScheme::NeutralText).color();
     case Theme::NegativeTextColor:
         return scheme->foreground(KColorScheme::NegativeText).color();
+    case Theme::DisabledTextColor:
+        return scheme->foreground(KColorScheme::InactiveText).color();
     }
 
     return QColor();
@@ -699,7 +702,7 @@ QColor ThemePrivate::color(Theme::ColorRole role, Theme::ColorGroup group) const
 
 void ThemePrivate::processWallpaperSettings(KConfigBase *metadata)
 {
-    if (!defaultWallpaperTheme.isEmpty() && defaultWallpaperTheme != QStringLiteral(DEFAULT_WALLPAPER_THEME)) {
+    if (!defaultWallpaperTheme.isEmpty() && defaultWallpaperTheme != QLatin1String(DEFAULT_WALLPAPER_THEME)) {
         return;
     }
 
@@ -784,7 +787,7 @@ void ThemePrivate::setThemeName(const QString &tempThemeName, bool writeSettings
     themeName = theme;
 
     // load the color scheme config
-    const QString colorsFile = realTheme ? QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1Literal(PLASMA_RELATIVE_DATA_INSTALL_DIR "/desktoptheme/") % theme % QLatin1Literal("/colors"))
+    const QString colorsFile = realTheme ? QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1String(PLASMA_RELATIVE_DATA_INSTALL_DIR "/desktoptheme/") % theme % QLatin1String("/colors"))
                                : QString();
 
     //qCDebug(LOG_PLASMA) << "we're going for..." << colorsFile << "*******************";
@@ -800,14 +803,15 @@ void ThemePrivate::setThemeName(const QString &tempThemeName, bool writeSettings
     buttonColorScheme = KColorScheme(QPalette::Active, KColorScheme::Button, colors);
     viewColorScheme = KColorScheme(QPalette::Active, KColorScheme::View, colors);
     complementaryColorScheme = KColorScheme(QPalette::Active, KColorScheme::Complementary, colors);
-    const QString wallpaperPath = QLatin1Literal(PLASMA_RELATIVE_DATA_INSTALL_DIR "/desktoptheme/") % theme % QLatin1Literal("/wallpapers/");
+    palette = KColorScheme::createApplicationPalette(colors);
+    const QString wallpaperPath = QLatin1String(PLASMA_RELATIVE_DATA_INSTALL_DIR "/desktoptheme/") % theme % QLatin1String("/wallpapers/");
     hasWallpapers = !QStandardPaths::locate(QStandardPaths::GenericDataLocation, wallpaperPath, QStandardPaths::LocateDirectory).isEmpty();
 
     // load the wallpaper settings, if any
     if (realTheme) {
-        const QString metadataPath(QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1Literal(PLASMA_RELATIVE_DATA_INSTALL_DIR "/desktoptheme/") % theme % QLatin1Literal("/metadata.desktop")));
+        const QString metadataPath(QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1String(PLASMA_RELATIVE_DATA_INSTALL_DIR "/desktoptheme/") % theme % QLatin1String("/metadata.desktop")));
         KConfig metadata(metadataPath, KConfig::SimpleConfig);
-        pluginInfo = KPluginInfo(metadataPath);
+        pluginMetaData = KPluginMetaData(metadataPath);
 
         processContrastSettings(&metadata);
         processBlurBehindSettings(&metadata);
@@ -821,7 +825,7 @@ void ThemePrivate::setThemeName(const QString &tempThemeName, bool writeSettings
         while (!fallback.isEmpty() && !fallbackThemes.contains(fallback)) {
             fallbackThemes.append(fallback);
 
-            QString metadataPath(QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1Literal(PLASMA_RELATIVE_DATA_INSTALL_DIR "/desktoptheme/") % fallback % QLatin1Literal("/metadata.desktop")));
+            QString metadataPath(QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1String(PLASMA_RELATIVE_DATA_INSTALL_DIR "/desktoptheme/") % fallback % QLatin1String("/metadata.desktop")));
             KConfig metadata(metadataPath, KConfig::SimpleConfig);
             KConfigGroup cg(&metadata, "Settings");
             fallback = cg.readEntry("FallbackTheme", QString());
@@ -832,7 +836,7 @@ void ThemePrivate::setThemeName(const QString &tempThemeName, bool writeSettings
         }
 
         foreach (const QString &theme, fallbackThemes) {
-            QString metadataPath(QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1Literal(PLASMA_RELATIVE_DATA_INSTALL_DIR "/desktoptheme/") % theme % QLatin1Literal("/metadata.desktop")));
+            QString metadataPath(QStandardPaths::locate(QStandardPaths::GenericDataLocation, QLatin1String(PLASMA_RELATIVE_DATA_INSTALL_DIR "/desktoptheme/") % theme % QLatin1String("/metadata.desktop")));
             KConfig metadata(metadataPath, KConfig::SimpleConfig);
             processWallpaperSettings(&metadata);
         }
