@@ -97,7 +97,7 @@ QVariantMap Service::operationDescription(const QString &operationName)
 ServiceJob *Service::startOperationCall(const QVariantMap &description, QObject *parent)
 {
     // TODO: nested groups?
-    ServiceJob *job = 0;
+    ServiceJob *job = nullptr;
     const QString op = !description.isEmpty() ? description.value(QStringLiteral("_name")).toString() : QString();
 
     if (d->operationsMap.isEmpty()) {
@@ -198,7 +198,7 @@ void Service::registerOperationsScheme()
         return;
     }
 
-    const QString path = QStandardPaths::locate(QStandardPaths::GenericDataLocation, PLASMA_RELATIVE_DATA_INSTALL_DIR "/services/" + d->name + ".operations");
+    const QString path = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral(PLASMA_RELATIVE_DATA_INSTALL_DIR "/services/") + d->name + QStringLiteral(".operations"));
 
     if (path.isEmpty()) {
 #ifndef NDEBUG
