@@ -32,6 +32,7 @@
 #include <QXmlStreamReader>
 #include <QXmlStreamWriter>
 #include <QBuffer>
+#include <QRegularExpression>
 
 #include <kcolorscheme.h>
 #include <kconfiggroup.h>
@@ -87,6 +88,7 @@ bool SharedSvgRenderer::load(
     // Apply the style sheet.
     if (!styleSheet.isEmpty() && contents.contains("current-color-scheme")) {
         QByteArray processedContents;
+        processedContents.reserve(contents.size());
         QXmlStreamReader reader(contents);
 
         QBuffer buffer(&processedContents);
@@ -117,19 +119,18 @@ bool SharedSvgRenderer::load(
 
     // Search the SVG to find and store all ids that contain size hints.
     const QString contentsAsString(QString::fromLatin1(contents));
-    QRegExp idExpr(QLatin1String("id\\s*=\\s*(['\"])(\\d+-\\d+-.*)\\1"));
-    idExpr.setMinimal(true);
+    static const QRegularExpression idExpr(QLatin1String("id\\s*?=\\s*?(['\"])(\\d+?-\\d+?-.*?)\\1"));
+    Q_ASSERT(idExpr.isValid());
 
-    int pos = 0;
-    while ((pos = idExpr.indexIn(contentsAsString, pos)) != -1) {
-        QString elementId = idExpr.cap(2);
+    auto matchIt = idExpr.globalMatch(contentsAsString);
+    while (matchIt.hasNext()) {
+        auto match = matchIt.next();
+        QString elementId = match.captured(2);
 
         QRectF elementRect = boundsOnElement(elementId);
         if (elementRect.isValid()) {
             interestingElements.insert(elementId, elementRect);
         }
-
-        pos += idExpr.matchedLength();
     }
 
     return true;
@@ -141,12 +142,12 @@ bool SharedSvgRenderer::load(
 
 SvgPrivate::SvgPrivate(Svg *svg)
     : q(svg),
-      renderer(0),
+      renderer(nullptr),
       styleCrc(0),
       colorGroup(Plasma::Theme::NormalColorGroup),
       lastModified(0),
       devicePixelRatio(1.0),
-      scaleFactor(1.0),
+      scaleFactor(s_lastScaleFactor),
       status(Svg::Status::Normal),
       multipleImages(false),
       themed(false),
@@ -185,14 +186,14 @@ bool SvgPrivate::setImagePath(const QString &imagePath)
     QString actualPath = imagePath;
     if (imagePath.startsWith(QLatin1String("file://"))) {
         //length of file://
-        actualPath = actualPath.mid(7);
+        actualPath.remove(0, 7);
     }
 
-    bool isThemed = !QDir::isAbsolutePath(actualPath);
+    bool isThemed = !actualPath.isEmpty() && !QDir::isAbsolutePath(actualPath);
     bool inIconTheme = false;
 
     //an absolute path.. let's try if this actually an *icon* theme
-    if (!isThemed) {
+    if (!isThemed && !actualPath.isEmpty()) {
         const auto *iconTheme = KIconLoader::global()->theme();
         isThemed = inIconTheme = iconTheme && actualPath.startsWith(iconTheme->dir());
     }
@@ -213,7 +214,7 @@ bool SvgPrivate::setImagePath(const QString &imagePath)
     if (isThemed && !themed && s_systemColorsCache) {
         // catch the case where we weren't themed, but now we are, and the colors cache was set up
         // ensure we are not connected to that theme previously
-        QObject::disconnect(s_systemColorsCache.data(), 0, q, 0);
+        QObject::disconnect(s_systemColorsCache.data(), nullptr, q, nullptr);
     }
 
     themed = isThemed;
@@ -222,7 +223,7 @@ bool SvgPrivate::setImagePath(const QString &imagePath)
     localRectCache.clear();
     elementsWithSizeHints.clear();
     bool oldFromCurrentTheme = fromCurrentTheme;
-    fromCurrentTheme = actualTheme()->currentThemeHasImage(imagePath);
+    fromCurrentTheme = !inIconTheme && isThemed && actualTheme()->currentThemeHasImage(imagePath);
 
     if (fromCurrentTheme != oldFromCurrentTheme) {
         emit q->fromCurrentThemeChanged(fromCurrentTheme);
@@ -237,7 +238,7 @@ bool SvgPrivate::setImagePath(const QString &imagePath)
         path = actualTheme()->imagePath(themePath);
         themeFailed = path.isEmpty();
         QObject::connect(actualTheme(), SIGNAL(themeChanged()), q, SLOT(themeChanged()));
-    } else if (QFile::exists(actualPath)) {
+    } else if (QFileInfo::exists(actualPath)) {
         QObject::connect(cacheAndColorsTheme(), SIGNAL(themeChanged()), q, SLOT(themeChanged()), Qt::UniqueConnection);
         path = actualPath;
     } else {
@@ -251,7 +252,7 @@ bool SvgPrivate::setImagePath(const QString &imagePath)
 
     // also images with absolute path needs to have a natural size initialized,
     // even if looks a bit weird using Theme to store non-themed stuff
-    if ((themed && QFile::exists(path)) || QFile::exists(actualPath)) {
+    if ((themed && !path.isEmpty() && QFileInfo::exists(path)) || QFileInfo::exists(actualPath)) {
         QRectF rect;
 
         if (cacheAndColorsTheme()->findInRectsCache(path, QStringLiteral("_Natural_%1").arg(scaleFactor), rect)) {
@@ -268,7 +269,7 @@ bool SvgPrivate::setImagePath(const QString &imagePath)
     if (!themed) {
         QFile f(actualPath);
         QFileInfo info(f);
-        lastModified = info.lastModified().toTime_t();
+        lastModified = info.lastModified().toSecsSinceEpoch();
     }
 
     q->resize();
@@ -309,16 +310,17 @@ QPixmap SvgPrivate::findInCache(const QString &elementId, qreal ratio, const QSi
     if (elementsWithSizeHints.isEmpty()) {
         // Fetch all size hinted element ids from the theme's rect cache
         // and store them locally.
-        QRegExp sizeHintedKeyExpr(CACHE_ID_NATURAL_SIZE("(\\d+)-(\\d+)-(.+)", status, ratio));
+        const QRegularExpression sizeHintedKeyExpr(QLatin1String("^") + CACHE_ID_NATURAL_SIZE(QStringLiteral("(\\d+)-(\\d+)-(.+)"), status, ratio) + QLatin1String("$"));
 
         foreach (const QString &key, cacheAndColorsTheme()->listCachedRectKeys(path)) {
-            if (sizeHintedKeyExpr.exactMatch(key)) {
-                QString baseElementId = sizeHintedKeyExpr.cap(3);
-                QSize sizeHint(sizeHintedKeyExpr.cap(1).toInt(),
-                               sizeHintedKeyExpr.cap(2).toInt());
+            const auto match = sizeHintedKeyExpr.match(key);
+            if (match.hasMatch()) {
+                QString baseElementId = match.captured(3);
+                QSize sizeHint(match.capturedRef(1).toInt(),
+                               match.capturedRef(2).toInt());
 
                 if (sizeHint.isValid()) {
-                    elementsWithSizeHints.insertMulti(baseElementId, sizeHint);
+                    elementsWithSizeHints.insert(baseElementId, sizeHint);
                 }
             }
         }
@@ -332,12 +334,12 @@ QPixmap SvgPrivate::findInCache(const QString &elementId, qreal ratio, const QSi
     // Look at the size hinted elements and try to find the smallest one with an
     // identical aspect ratio.
     if (s.isValid() && !elementId.isEmpty()) {
-        QList<QSize> elementSizeHints = elementsWithSizeHints.values(elementId);
+        const QList<QSize> elementSizeHints = elementsWithSizeHints.values(elementId);
 
         if (!elementSizeHints.isEmpty()) {
             QSize bestFit(-1, -1);
 
-            Q_FOREACH (QSize hint, elementSizeHints) {
+            for (const QSize &hint : elementSizeHints) {
 
                 if (hint.width() >= s.width() * ratio && hint.height() >= s.height() * ratio &&
                         (!bestFit.isValid() ||
@@ -347,8 +349,8 @@ QPixmap SvgPrivate::findInCache(const QString &elementId, qreal ratio, const QSi
             }
 
             if (bestFit.isValid()) {
-                actualElementId = QString::number(bestFit.width()) % '-' %
-                                  QString::number(bestFit.height()) % '-' % elementId;
+                actualElementId = QString::number(bestFit.width()) % QLatin1Char('-') %
+                                  QString::number(bestFit.height()) % QLatin1Char('-') % elementId;
             }
         }
     }
@@ -367,11 +369,7 @@ QPixmap SvgPrivate::findInCache(const QString &elementId, qreal ratio, const QSi
         return QPixmap();
     }
 
-    QString id = cachePath(path, size);
-
-    if (!actualElementId.isEmpty()) {
-        id.append(actualElementId);
-    }
+    const QString id = cachePath(path, size) + actualElementId;
 
     //qCDebug(LOG_PLASMA) << "id is " << id;
 
@@ -456,7 +454,7 @@ void SvgPrivate::createRenderer()
     //qCDebug(LOG_PLASMA) << path << "**";
 
     QString styleSheet = cacheAndColorsTheme()->d->svgStyleSheet(colorGroup, status);
-    styleCrc = qChecksum(styleSheet.toUtf8(), styleSheet.size());
+    styleCrc = qChecksum(styleSheet.toUtf8().constData(), styleSheet.size());
 
     QHash<QString, SharedSvgRenderer::Ptr>::const_iterator it = s_renderers.constFind(styleCrc + path);
 
@@ -503,7 +501,7 @@ void SvgPrivate::eraseRenderer()
         }
     }
 
-    renderer = 0;
+    renderer = nullptr;
     styleCrc = 0;
     localRectCache.clear();
     elementsWithSizeHints.clear();
@@ -528,37 +526,37 @@ QRectF SvgPrivate::elementRect(const QString &elementId)
         return QRectF();
     }
 
-    QString id = cacheId(elementId);
-
-    if (localRectCache.contains(id)) {
-        return localRectCache.value(id);
+    const QString id = cacheId(elementId);
+    const auto it = localRectCache.constFind(id);
+    if (it != localRectCache.constEnd()) {
+        return *it;
     }
 
     QRectF rect;
     bool found = cacheAndColorsTheme()->findInRectsCache(path, id, rect);
-
     //This is a corner case where we are *sure* the element is not valid
     if (found && rect == QRectF()) {
         return rect;
     } else if (found) {
         localRectCache.insert(id, rect);
     } else {
-        rect = findAndCacheElementRect(elementId);
+        rect = findAndCacheElementRect(elementId, id);
     }
 
     return rect;
 }
 
-QRectF SvgPrivate::findAndCacheElementRect(const QString &elementId)
+QRectF SvgPrivate::findAndCacheElementRect(const QString &elementId, const QString &id)
 {
+    //we need to check the id before createRenderer(), otherwise it may generate a different id compared to the previous cacheId)( call
     createRenderer();
 
-    // createRenderer() can insert some interesting rects in the cache, so check it
-    const QString id = cacheId(elementId);
-    if (localRectCache.contains(id)) {
-        return localRectCache.value(id);
+    const auto it = localRectCache.constFind(id);
+    if (it != localRectCache.constEnd()) {
+        return *it;
     }
 
+    //This code will usually never be run because createRenderer already caches all the boundingRect in the elements in the svg
     QRectF elementRect = renderer->elementExists(elementId) ?
                          renderer->matrixForElement(elementId).map(renderer->boundsOnElement(elementId)).boundingRect() :
                          QRectF();
@@ -571,6 +569,7 @@ QRectF SvgPrivate::findAndCacheElementRect(const QString &elementId)
                          elementRect.width() * dx, elementRect.height() * dy);
 
     cacheAndColorsTheme()->insertIntoRectsCache(path, id, elementRect);
+    localRectCache.insert(id, elementRect);
 
     return elementRect;
 }
@@ -694,7 +693,8 @@ void SvgPrivate::colorsChanged()
 }
 
 QHash<QString, SharedSvgRenderer::Ptr> SvgPrivate::s_renderers;
-QWeakPointer<Theme> SvgPrivate::s_systemColorsCache;
+QPointer<Theme> SvgPrivate::s_systemColorsCache;
+qreal SvgPrivate::s_lastScaleFactor = 1.0;
 
 Svg::Svg(QObject *parent)
     : QObject(parent),
@@ -741,6 +741,7 @@ void Svg::setScaleFactor(qreal ratio)
     }
 
     d->scaleFactor = floor(ratio);
+    d->s_lastScaleFactor = d->scaleFactor;
     //not resize() because we want to do it unconditionally
     QRectF rect;
 
@@ -769,7 +770,7 @@ void Svg::setColorGroup(Plasma::Theme::ColorGroup group)
     }
 
     d->colorGroup = group;
-    d->renderer = 0;
+    d->renderer = nullptr;
     emit colorGroupChanged();
     emit repaintNeeded();
 }
@@ -899,7 +900,7 @@ bool Svg::isValid() const
         return true;
     }
 
-    if (!QFile::exists(d->path)) {
+    if (d->path.isEmpty() || !QFileInfo::exists(d->path)) {
         return false;
     }
 
@@ -967,7 +968,7 @@ void Svg::setTheme(Plasma::Theme *theme)
     }
 
     if (d->theme) {
-        disconnect(d->theme.data(), 0, this, 0);
+        disconnect(d->theme.data(), nullptr, this, nullptr);
     }
 
     d->theme = theme;
