@@ -73,13 +73,10 @@ void ContainmentViewPrivate::setContainment(Plasma::Containment *cont)
     Plasma::Types::FormFactor oldForm = formFactor();
 
     if (containment) {
-        QObject::disconnect(containment, 0, q, 0);
+        QObject::disconnect(containment, nullptr, q, nullptr);
         QObject *oldGraphicObject = containment->property("_plasma_graphicObject").value<QObject *>();
-        if (oldGraphicObject) {
-//             qDebug() << "Old graphics Object:" << oldGraphicObject << "Old containment" << containment.data();
-            //make sure the graphic object won't die with us
-            //FIXME:we need a way to reparent to *NO* graphics item, but this makes Qt crash
-            oldGraphicObject->setParent(containment);
+        if (auto item = qobject_cast<QQuickItem*>(oldGraphicObject)) {
+            item->setVisible(false);
         }
         containment->reactToScreenChange();
     }
@@ -97,7 +94,7 @@ void ContainmentViewPrivate::setContainment(Plasma::Containment *cont)
 
     //we are QuickViewSharedEngine::SizeRootObjectToView, but that's not enough, as
     //the root object isn't immediately resized (done at the resizeEvent handler).
-    //by resising it just before restoring the containment, it removes a chain of resizes at startup
+    //by resizing it just before restoring the containment, it removes a chain of resizes at startup
     if (q->rootObject()) {
         q->rootObject()->setSize(q->size());
     }
@@ -137,8 +134,7 @@ void ContainmentViewPrivate::setContainment(Plasma::Containment *cont)
 
         graphicObject->setFocus(true);
         //by resizing before adding, it will avoid some resizes in most cases
-        graphicObject->setProperty("width", q->width());
-        graphicObject->setProperty("height", q->height());
+        graphicObject->setSize(q->size());
         graphicObject->setParentItem(q->rootObject());
         if (q->rootObject()) {
             q->rootObject()->setProperty("containment", QVariant::fromValue(graphicObject));
@@ -177,6 +173,7 @@ void ContainmentViewPrivate::showConfigurationInterface(Plasma::Applet *applet)
             configContainmentView->hide();
             configContainmentView->deleteLater();
         } else {
+            configContainmentView->raise();
             configContainmentView->requestActivate();
             return;
         }
@@ -209,7 +206,7 @@ ContainmentView::ContainmentView(Plasma::Corona *corona, QWindow *parent)
     if (corona->kPackage().isValid()) {
         const auto info = corona->kPackage().metadata();
         if (info.isValid()) {
-            setTranslationDomain("plasma_shell_" + info.pluginId());
+            setTranslationDomain(QStringLiteral("plasma_shell_") + info.pluginId());
         } else {
             qWarning() << "Invalid corona package metadata";
         }
