@@ -18,11 +18,11 @@
 
 #include "dialogshadows_p.h"
 
-#include <QGlobalStatic>
 #include <QWindow>
 #include <QPainter>
 #include <config-plasma.h>
 
+#include <KWindowSystem>
 #if HAVE_X11
 #include <QX11Info>
 #include <X11/Xatom.h>
@@ -31,6 +31,13 @@
 #include <fixx11h.h>
 #endif
 
+#if HAVE_KWAYLAND
+#include "waylandintegration_p.h"
+#include <KWayland/Client/shadow.h>
+#include <KWayland/Client/shm_pool.h>
+#include <KWayland/Client/surface.h>
+#endif
+
 #include <qdebug.h>
 
 class DialogShadows::Private
@@ -39,9 +46,9 @@ public:
     Private(DialogShadows *shadows)
         : q(shadows)
 #if HAVE_X11
-        , _connection(0x0),
-        _gc(0x0)
-        , m_isX11(QX11Info::isPlatformX11())
+        , _connection(nullptr)
+        , _gc(0x0)
+        , m_isX11(KWindowSystem::isPlatformX11())
 #endif
     {
     }
@@ -54,13 +61,18 @@ public:
     }
 
     void freeX11Pixmaps();
+    void freeWaylandBuffers();
     void clearPixmaps();
     void setupPixmaps();
     Qt::HANDLE createPixmap(const QPixmap &source);
     void initPixmap(const QString &element);
     QPixmap initEmptyPixmap(const QSize &size);
     void updateShadow(const QWindow *window, Plasma::FrameSvg::EnabledBorders);
+    void updateShadowX11(const QWindow *window, Plasma::FrameSvg::EnabledBorders);
+    void updateShadowWayland(const QWindow *window, Plasma::FrameSvg::EnabledBorders);
     void clearShadow(const QWindow *window);
+    void clearShadowX11(const QWindow *window);
+    void clearShadowWayland(const QWindow *window);
     void updateShadows();
     void windowDestroyed(QObject *deletedObject);
     void setupData(Plasma::FrameSvg::EnabledBorders enabledBorders);
@@ -85,6 +97,13 @@ public:
     bool m_isX11;
 #endif
 
+#if HAVE_KWAYLAND
+    struct Wayland {
+        QList<KWayland::Client::Buffer::Ptr> shadowBuffers;
+    };
+    Wayland m_wayland;
+#endif
+
     QHash<Plasma::FrameSvg::EnabledBorders, QVector<unsigned long> > data;
     QHash<const QWindow *, Plasma::FrameSvg::EnabledBorders> m_windows;
 };
@@ -138,7 +157,7 @@ void DialogShadows::removeWindow(const QWindow *window)
     }
 
     d->m_windows.remove(window);
-    disconnect(window, 0, this, 0);
+    disconnect(window, nullptr, this, nullptr);
     d->clearShadow(window);
 
     if (d->m_windows.isEmpty()) {
@@ -146,6 +165,16 @@ void DialogShadows::removeWindow(const QWindow *window)
     }
 }
 
+void DialogShadows::setEnabledBorders(const QWindow *window, Plasma::FrameSvg::EnabledBorders enabledBorders)
+{
+    if (!window || !d->m_windows.contains(window)) {
+        return;
+    }
+
+    d->updateShadow(window, enabledBorders);
+}
+
+
 void DialogShadows::Private::windowDestroyed(QObject *deletedObject)
 {
     m_windows.remove(static_cast<QWindow *>(deletedObject));
@@ -169,7 +198,7 @@ Qt::HANDLE DialogShadows::Private::createPixmap(const QPixmap &source)
 
     // do nothing for invalid pixmaps
     if (source.isNull()) {
-        return 0;
+        return nullptr;
     }
 
     /*
@@ -180,7 +209,7 @@ Qt::HANDLE DialogShadows::Private::createPixmap(const QPixmap &source)
 
 #if HAVE_X11
     if (!m_isX11) {
-        return 0;
+        return nullptr;
     }
 
     // check connection
@@ -197,7 +226,7 @@ Qt::HANDLE DialogShadows::Private::createPixmap(const QPixmap &source)
     // check gc
     if (!_gc) {
         _gc = xcb_generate_id(_connection);
-        xcb_create_gc(_connection, _gc, pixmap, 0, 0x0);
+        xcb_create_gc(_connection, _gc, pixmap, 0, nullptr);
     }
 
 //         // create explicitly shared QPixmap from it
@@ -217,12 +246,12 @@ Qt::HANDLE DialogShadows::Private::createPixmap(const QPixmap &source)
         _connection, XCB_IMAGE_FORMAT_Z_PIXMAP, pixmap, _gc,
         image.width(), image.height(), 0, 0,
         0, 32,
-        image.byteCount(), image.constBits());
+        image.sizeInBytes(), image.constBits());
 
     return (Qt::HANDLE)pixmap;
 
 #else
-    return 0;
+    return nullptr;
 #endif
 
 }
@@ -269,6 +298,13 @@ void DialogShadows::Private::setupPixmaps()
     m_emptyVerticalPix = initEmptyPixmap(QSize(1, q->elementSize(QStringLiteral("shadow-left")).height()));
     m_emptyHorizontalPix = initEmptyPixmap(QSize(q->elementSize(QStringLiteral("shadow-top")).width(), 1));
 
+#if HAVE_KWAYLAND
+    if (KWayland::Client::ShmPool *shmPool = WaylandIntegration::self()->waylandShmPool()) {
+        for (auto it = m_shadowPixmaps.constBegin(); it != m_shadowPixmaps.constEnd(); ++it) {
+            m_wayland.shadowBuffers << shmPool->createBuffer(it->toImage());
+        }
+    }
+#endif
 }
 
 void DialogShadows::Private::setupData(Plasma::FrameSvg::EnabledBorders enabledBorders)
@@ -410,35 +446,38 @@ void DialogShadows::Private::freeX11Pixmaps()
     if (!m_isX11) {
         return;
     }
+
+    auto *display = QX11Info::display();
+    if (!display) {
+        return;
+    }
+
     foreach (const QPixmap &pixmap, m_shadowPixmaps) {
-        if (!QX11Info::display()) {
-            return;
-        }
         if (!pixmap.isNull()) {
-            XFreePixmap(QX11Info::display(), reinterpret_cast<unsigned long>(createPixmap(pixmap)));
+            XFreePixmap(display, reinterpret_cast<unsigned long>(createPixmap(pixmap)));
         }
     }
 
     if (!m_emptyCornerPix.isNull()) {
-        XFreePixmap(QX11Info::display(), reinterpret_cast<unsigned long>(createPixmap(m_emptyCornerPix)));
+        XFreePixmap(display, reinterpret_cast<unsigned long>(createPixmap(m_emptyCornerPix)));
     }
     if (!m_emptyCornerBottomPix.isNull()) {
-        XFreePixmap(QX11Info::display(), reinterpret_cast<unsigned long>(createPixmap(m_emptyCornerBottomPix)));
+        XFreePixmap(display, reinterpret_cast<unsigned long>(createPixmap(m_emptyCornerBottomPix)));
     }
     if (!m_emptyCornerLeftPix.isNull()) {
-        XFreePixmap(QX11Info::display(), reinterpret_cast<unsigned long>(createPixmap(m_emptyCornerLeftPix)));
+        XFreePixmap(display, reinterpret_cast<unsigned long>(createPixmap(m_emptyCornerLeftPix)));
     }
     if (!m_emptyCornerRightPix.isNull()) {
-        XFreePixmap(QX11Info::display(), reinterpret_cast<unsigned long>(createPixmap(m_emptyCornerRightPix)));
+        XFreePixmap(display, reinterpret_cast<unsigned long>(createPixmap(m_emptyCornerRightPix)));
     }
     if (!m_emptyCornerTopPix.isNull()) {
-        XFreePixmap(QX11Info::display(), reinterpret_cast<unsigned long>(createPixmap(m_emptyCornerTopPix)));
+        XFreePixmap(display, reinterpret_cast<unsigned long>(createPixmap(m_emptyCornerTopPix)));
     }
     if (!m_emptyVerticalPix.isNull()) {
-        XFreePixmap(QX11Info::display(), reinterpret_cast<unsigned long>(createPixmap(m_emptyVerticalPix)));
+        XFreePixmap(display, reinterpret_cast<unsigned long>(createPixmap(m_emptyVerticalPix)));
     }
     if (!m_emptyHorizontalPix.isNull()) {
-        XFreePixmap(QX11Info::display(), reinterpret_cast<unsigned long>(createPixmap(m_emptyHorizontalPix)));
+        XFreePixmap(display, reinterpret_cast<unsigned long>(createPixmap(m_emptyHorizontalPix)));
     }
 #endif
 }
@@ -456,16 +495,35 @@ void DialogShadows::Private::clearPixmaps()
     m_emptyVerticalPix = QPixmap();
     m_emptyHorizontalPix = QPixmap();
 #endif
+    freeWaylandBuffers();
     m_shadowPixmaps.clear();
     data.clear();
 }
 
+void DialogShadows::Private::freeWaylandBuffers()
+{
+#if HAVE_KWAYLAND
+    m_wayland.shadowBuffers.clear();
+#endif
+}
+
 void DialogShadows::Private::updateShadow(const QWindow *window, Plasma::FrameSvg::EnabledBorders enabledBorders)
 {
 #if HAVE_X11
-    if (!m_isX11) {
-        return;
+    if (m_isX11) {
+        updateShadowX11(window, enabledBorders);
     }
+#endif
+#if HAVE_KWAYLAND
+    if (WaylandIntegration::self()->waylandShadowManager()) {
+        updateShadowWayland(window, enabledBorders);
+    }
+#endif
+}
+
+void DialogShadows::Private::updateShadowX11(const QWindow *window, Plasma::FrameSvg::EnabledBorders enabledBorders)
+{
+#if HAVE_X11
     if (m_shadowPixmaps.isEmpty()) {
         setupPixmaps();
     }
@@ -480,23 +538,151 @@ void DialogShadows::Private::updateShadow(const QWindow *window, Plasma::FrameSv
     //qDebug() << "going to set the shadow of" << window->winId() << "to" << data;
     XChangeProperty(dpy, window->winId(), atom, XA_CARDINAL, 32, PropModeReplace,
                     reinterpret_cast<const unsigned char *>(data[enabledBorders].constData()), data[enabledBorders].size());
-#else
-    Q_UNUSED(window)
-    Q_UNUSED(enabledBorders)
+#endif
+}
+
+void DialogShadows::Private::updateShadowWayland(const QWindow *window, Plasma::FrameSvg::EnabledBorders enabledBorders)
+{
+#if HAVE_KWAYLAND
+    if (!WaylandIntegration::self()->waylandShmPool()) {
+        return;
+    }
+    if (m_wayland.shadowBuffers.isEmpty()) {
+        setupPixmaps();
+    }
+    // TODO: check whether the surface already has a shadow
+    KWayland::Client::Surface *surface = KWayland::Client::Surface::fromWindow(const_cast<QWindow*>(window));
+    if (!surface) {
+        return;
+    }
+    KWayland::Client::ShadowManager *manager = WaylandIntegration::self()->waylandShadowManager();
+    auto shadow = manager->createShadow(surface, surface);
+
+    //shadow-top
+    if (enabledBorders & Plasma::FrameSvg::TopBorder) {
+        shadow->attachTop(m_wayland.shadowBuffers.at(0));
+    }
+
+    //shadow-topright
+    if (enabledBorders & Plasma::FrameSvg::TopBorder &&
+        enabledBorders & Plasma::FrameSvg::RightBorder) {
+        shadow->attachTopRight(m_wayland.shadowBuffers.at(1));
+    }
+
+    //shadow-right
+    if (enabledBorders & Plasma::FrameSvg::RightBorder) {
+        shadow->attachRight(m_wayland.shadowBuffers.at(2));
+    }
+
+    //shadow-bottomright
+    if (enabledBorders & Plasma::FrameSvg::BottomBorder &&
+        enabledBorders & Plasma::FrameSvg::RightBorder) {
+        shadow->attachBottomRight(m_wayland.shadowBuffers.at(3));
+    }
+
+    //shadow-bottom
+    if (enabledBorders & Plasma::FrameSvg::BottomBorder) {
+        shadow->attachBottom(m_wayland.shadowBuffers.at(4));
+    }
+
+    //shadow-bottomleft
+    if (enabledBorders & Plasma::FrameSvg::BottomBorder &&
+        enabledBorders & Plasma::FrameSvg::LeftBorder) {
+        shadow->attachBottomLeft(m_wayland.shadowBuffers.at(5));
+    }
+
+    //shadow-left
+    if (enabledBorders & Plasma::FrameSvg::LeftBorder) {
+        shadow->attachLeft(m_wayland.shadowBuffers.at(6));
+    }
+
+    //shadow-topleft
+    if (enabledBorders & Plasma::FrameSvg::TopBorder &&
+        enabledBorders & Plasma::FrameSvg::LeftBorder) {
+        shadow->attachTopLeft(m_wayland.shadowBuffers.at(7));
+    }
+
+    QSize marginHint;
+    QMarginsF margins;
+    if (enabledBorders & Plasma::FrameSvg::TopBorder) {
+        marginHint = q->elementSize(QStringLiteral("shadow-hint-top-margin"));
+        if (marginHint.isValid()) {
+            margins.setTop(marginHint.height());
+        } else {
+            margins.setTop(m_shadowPixmaps[0].height());
+        }
+    }
+
+    if (enabledBorders & Plasma::FrameSvg::RightBorder) {
+        marginHint = q->elementSize(QStringLiteral("shadow-hint-right-margin"));
+        if (marginHint.isValid()) {
+            margins.setRight(marginHint.width());
+        } else {
+            margins.setRight(m_shadowPixmaps[2].width());
+        }
+    }
+
+    if (enabledBorders & Plasma::FrameSvg::BottomBorder) {
+        marginHint = q->elementSize(QStringLiteral("shadow-hint-bottom-margin"));
+        if (marginHint.isValid()) {
+            margins.setBottom(marginHint.height());
+        } else {
+            margins.setBottom(m_shadowPixmaps[4].height());
+        }
+    }
+
+    if (enabledBorders & Plasma::FrameSvg::LeftBorder) {
+        marginHint = q->elementSize(QStringLiteral("shadow-hint-left-margin"));
+        if (marginHint.isValid()) {
+            margins.setLeft(marginHint.width());
+        } else {
+            margins.setLeft(m_shadowPixmaps[6].width());
+        }
+    }
+
+    shadow->setOffsets(margins);
+    shadow->commit();
+    surface->commit(KWayland::Client::Surface::CommitFlag::None);
 #endif
 }
 
 void DialogShadows::Private::clearShadow(const QWindow *window)
 {
-#if HAVE_X11
-    if (!m_isX11) {
+    if (!static_cast<const QSurface*>(window)->surfaceHandle()) {
+        qWarning() << "Cannot clear shadow from window without native surface!";
         return;
     }
+#if HAVE_X11
+    if (m_isX11) {
+        clearShadowX11(window);
+    }
+#endif
+#if HAVE_KWAYLAND
+    if (WaylandIntegration::self()->waylandShadowManager()) {
+        clearShadowWayland(window);
+    }
+#endif
+}
+
+void DialogShadows::Private::clearShadowX11(const QWindow* window)
+{
+#if HAVE_X11
     Display *dpy = QX11Info::display();
     Atom atom = XInternAtom(dpy, "_KDE_NET_WM_SHADOW", False);
     XDeleteProperty(dpy, window->winId(), atom);
-#else
-    Q_UNUSED(window)
+#endif
+}
+
+void DialogShadows::Private::clearShadowWayland(const QWindow *window)
+{
+#if HAVE_KWAYLAND
+    KWayland::Client::Surface *surface = KWayland::Client::Surface::fromWindow(const_cast<QWindow*>(window));
+    if (!surface) {
+        return;
+    }
+    KWayland::Client::ShadowManager *manager = WaylandIntegration::self()->waylandShadowManager();
+    manager->removeShadow(surface);
+    surface->commit(KWayland::Client::Surface::CommitFlag::None);
 #endif
 }
 
