@@ -21,6 +21,7 @@
 
 #include "containmentinterface.h"
 #include "wallpaperinterface.h"
+#include "dropmenu.h"
 #include <kdeclarative/qmlobject.h>
 
 #include <QClipboard>
@@ -28,6 +29,8 @@
 #include <QQmlProperty>
 #include <QMimeData>
 #include <QVersionNumber>
+#include <QScreen>
+#include <QApplication>
 
 #include <kactioncollection.h>
 #include <KAcceleratorManager>
@@ -59,8 +62,7 @@ ContainmentInterface::ContainmentInterface(DeclarativeAppletScript *parent, cons
     : AppletInterface(parent, args),
       m_wallpaperInterface(nullptr),
       m_activityInfo(nullptr),
-      m_wheelDelta(0),
-      m_editMode(false)
+      m_wheelDelta(0)
 {
     m_containment = static_cast<Plasma::Containment *>(appletScript()->applet()->containment());
 
@@ -71,6 +73,9 @@ ContainmentInterface::ContainmentInterface(DeclarativeAppletScript *parent, cons
     connect(m_containment.data(), &Plasma::Containment::appletAdded,
             this, &ContainmentInterface::appletAddedForward);
 
+    connect(m_containment->corona(), &Plasma::Corona::editModeChanged,
+            this, &ContainmentInterface::editModeChanged);
+
     if (!m_appletInterfaces.isEmpty()) {
         emit appletsChanged();
     }
@@ -87,6 +92,10 @@ void ContainmentInterface::init()
             this, &ContainmentInterface::activityNameChanged);
     emit activityNameChanged();
 
+    if (!m_containment->wallpaper().isEmpty()) {
+        loadWallpaper();
+    }
+
     AppletInterface::init();
 
     //Create the ToolBox
@@ -144,13 +153,9 @@ void ContainmentInterface::init()
         prop.write(expr.evaluate());
     }
 
-    if (!m_containment->wallpaper().isEmpty()) {
-        loadWallpaper();
-    }
-
     connect(m_containment.data(), &Plasma::Containment::activityChanged,
             this, &ContainmentInterface::activityChanged);
-    connect(m_containment.data(), &Plasma::Containment::activityChanged,
+    connect(m_containment.data(), &Plasma::Containment::activityChanged, this,
             [ = ]() {
                 delete m_activityInfo;
                 m_activityInfo = new KActivities::Info(m_containment->activity(), this);
@@ -163,17 +168,8 @@ void ContainmentInterface::init()
     connect(m_containment.data(), &Plasma::Containment::containmentTypeChanged,
             this, &ContainmentInterface::containmentTypeChanged);
 
-    connect(m_containment.data()->actions(), &KActionCollection::inserted,
+    connect(m_containment.data()->actions(), &KActionCollection::changed,
             this, &ContainmentInterface::actionsChanged);
-    connect(m_containment.data()->actions(), &KActionCollection::removed,
-            this, &ContainmentInterface::actionsChanged);
-
-    if (m_containment->corona()) {
-        connect(m_containment->corona(), &Plasma::Corona::availableScreenRegionChanged,
-                this, &ContainmentInterface::availableScreenRegionChanged);
-        connect(m_containment->corona(), &Plasma::Corona::availableScreenRectChanged,
-                this, &ContainmentInterface::availableScreenRectChanged);
-    }
 }
 
 QList <QObject *> ContainmentInterface::applets()
@@ -408,23 +404,19 @@ QPointF ContainmentInterface::adjustToAvailableScreenRegion(int x, int y, int w,
     return rect.topLeft();
 }
 
+QAction *ContainmentInterface::globalAction(QString name) const
+{
+    return m_containment->corona()->actions()->action(name);
+}
+
 bool ContainmentInterface::isEditMode() const
 {
-    return m_editMode;
+    return m_containment->corona()->isEditMode();
 }
 
 void ContainmentInterface::setEditMode(bool edit)
 {
-    if (edit == m_editMode) {
-        return;
-    }
-
-    if (m_containment->immutability() != Plasma::Types::Mutable) {
-        return;
-    }
-
-    m_editMode = edit;
-    emit editModeChanged();
+    m_containment->corona()->setEditMode(edit);
 }
 
 void ContainmentInterface::processMimeData(QObject *mimeDataProxy, int x, int y, KIO::DropJob *dropJob)
@@ -443,6 +435,17 @@ void ContainmentInterface::processMimeData(QMimeData *mimeData, int x, int y, KI
         return;
     }
 
+    if (m_dropMenu) {
+        if (dropJob) {
+            dropJob->kill();
+        }
+        return;
+    }
+    m_dropMenu = QPointer<DropMenu>(new DropMenu(dropJob, QPoint(x, y), this));
+    if (dropJob) {
+        dropJob->setParent(m_dropMenu);
+    }
+
     //const QMimeData *mimeData = data;
 
     qDebug() << "Arrived mimeData" << mimeData->urls() << mimeData->formats() << "at" << x << ", " << y;
@@ -461,42 +464,38 @@ void ContainmentInterface::processMimeData(QMimeData *mimeData, int x, int y, KI
 
             metaObject()->invokeMethod(this, "createApplet", Qt::QueuedConnection, Q_ARG(QString, appletName), Q_ARG(QVariantList, QVariantList()), Q_ARG(QRectF, QRectF(x, y, -1, -1)));
         }
+        delete m_dropMenu.data();
     } else if (mimeData->hasUrls()) {
         //TODO: collect the mimetypes of available script engines and offer
         //      to create widgets out of the matching URLs, if any
         const QList<QUrl> urls = KUrlMimeData::urlsFromMimeData(mimeData);
+        m_dropMenu->setUrls(urls);
+
+        if (!urls.at(0).isLocalFile()) {
+            QApplication::setOverrideCursor(Qt::WaitCursor);
+        }
+
+        QMimeDatabase db;
+        QMimeType firstMimetype = db.mimeTypeForUrl(urls.at(0));
         foreach (const QUrl &url, urls) {
-            QMimeDatabase db;
-            const QMimeType &mime = db.mimeTypeForUrl(url);
-            QString mimeName = mime.name();
-            QVariantList args;
-            args << url.url();
-            qDebug() << "can decode" << mimeName << args;
-
-            // It may be a directory or a file, let's stat
-            KIO::JobFlags flags = KIO::HideProgressInfo;
-            KIO::MimetypeJob *job = KIO::mimetype(url, flags);
-            m_dropPoints[job] = QPoint(x, y);
-
-            QObject::connect(job, SIGNAL(result(KJob*)), this, SLOT(dropJobResult(KJob*)));
-            QObject::connect(job, SIGNAL(mimetype(KIO::Job*,QString)),
-                             this, SLOT(mimeTypeRetrieved(KIO::Job*,QString)));
-
-            if (dropJob) {
-                m_dropJobs[job] = dropJob;
-            } else {
-                QMenu *choices = new QMenu(i18n("Content dropped"));
-                if (choices->winId()) {
-                    choices->windowHandle()->setTransientParent(window());
-                }
-                choices->addAction(QIcon::fromTheme(QStringLiteral("process-working")), i18n("Fetching file type..."));
-                choices->popup(window() ? window()->mapToGlobal(QPoint(x, y)) : QPoint(x, y));
-
-                m_dropMenus[job] = choices;
+            if (firstMimetype != db.mimeTypeForUrl(url)) {
+                m_dropMenu->setMultipleMimetypes(true);
+                break;
             }
         }
 
+        // It may be a directory or a file, let's stat
+        KIO::JobFlags flags = KIO::HideProgressInfo;
+        KIO::MimetypeJob *job = KIO::mimetype(m_dropMenu->urls().at(0), flags);
+        job->setParent(m_dropMenu.data());
+
+        QObject::connect(job, &KJob::result, this, &ContainmentInterface::dropJobResult);
+        QObject::connect(job, SIGNAL(mimetype(KIO::Job*, QString)),
+                this, SLOT(mimeTypeRetrieved(KIO::Job*,QString)));
+
     } else {
+        bool deleteDropMenu = true;
+
         QStringList formats = mimeData->formats();
         QHash<QString, KPluginMetaData> seenPlugins;
         QHash<QString, QString> pluginFormats;
@@ -518,33 +517,22 @@ void ContainmentInterface::processMimeData(QMimeData *mimeData, int x, int y, KI
         QString selectedPlugin;
 
         if (seenPlugins.isEmpty()) {
-            // do nothing
+        //do nothing
         //directly create if only one offer only if the containment didn't pass an existing plugin
         } else if (seenPlugins.count() == 1) {
             selectedPlugin = seenPlugins.constBegin().key();
             Plasma::Applet *applet = createApplet(selectedPlugin, QVariantList(), QRect(x, y, -1, -1));
             setAppletArgs(applet, pluginFormats[selectedPlugin], QString::fromUtf8(mimeData->data(pluginFormats[selectedPlugin])));
         } else {
-            QMenu *choices = nullptr;
-            if (!dropJob) {
-                choices = new QMenu();
-                if (choices->winId()) {
-                    choices->windowHandle()->setTransientParent(window());
-                }
-            }
-            QList<QAction *> extraActions;
             QHash<QAction *, QString> actionsToPlugins;
             foreach (const auto &info, seenPlugins) {
                 QAction *action;
                 if (!info.iconName().isEmpty()) {
-                    action = new QAction(QIcon::fromTheme(info.iconName()), info.name(), nullptr);
+                    action = new QAction(QIcon::fromTheme(info.iconName()), info.name(), m_dropMenu);
                 } else {
-                    action = new QAction(info.name(), nullptr);
-                }
-                extraActions << action;
-                if (choices) {
-                    choices->addAction(action);
+                    action = new QAction(info.name(), m_dropMenu);
                 }
+                m_dropMenu->addAction(action);
                 action->setData(info.pluginId());
                 connect(action, &QAction::triggered, this, [this, x, y, mimeData, action]() {
                     const QString selectedPlugin = action->data().toString();
@@ -554,16 +542,13 @@ void ContainmentInterface::processMimeData(QMimeData *mimeData, int x, int y, KI
 
                 actionsToPlugins.insert(action, info.pluginId());
             }
+            m_dropMenu->show();
+            deleteDropMenu = false;
+        }
 
-            //if the menu was created by ourselves, delete it
-            if (choices) {
-                QAction *choice = choices->exec(window() ? window()->mapToGlobal(QPoint(x, y)) : QPoint(x, y));
-
-                delete choices;
-            } else {
-                Q_ASSERT(dropJob);
-                dropJob->setApplicationActions(extraActions);
-            }
+        if (deleteDropMenu) {
+            // in case m_dropMenu has not been shown
+            delete m_dropMenu.data();
         }
     }
 }
@@ -571,16 +556,20 @@ void ContainmentInterface::processMimeData(QMimeData *mimeData, int x, int y, KI
 void ContainmentInterface::clearDataForMimeJob(KIO::Job *job)
 {
     QObject::disconnect(job, nullptr, this, nullptr);
-    m_dropPoints.remove(job);
-    QMenu *choices = m_dropMenus.take(job);
-    m_dropJobs.remove(job);
     job->kill();
+
+    m_dropMenu->show();
+
+    if (!m_dropMenu->urls().at(0).isLocalFile()) {
+        QApplication::restoreOverrideCursor();
+    }
 }
 
 void ContainmentInterface::dropJobResult(KJob *job)
 {
     if (job->error()) {
         qDebug() << "ERROR" << job->error() << ' ' << job->errorString();
+        clearDataForMimeJob(dynamic_cast<KIO::Job *>(job));
     }
 }
 
@@ -594,6 +583,7 @@ void ContainmentInterface::mimeTypeRetrieved(KIO::Job *job, const QString &mimet
         clearDataForMimeJob(job);
         return;
     }
+
     QList<KPluginMetaData> appletList = Plasma::PluginLoader::self()->listAppletMetaDataForUrl(tjob->url());
     if (mimetype.isEmpty() && appletList.isEmpty()) {
         clearDataForMimeJob(job);
@@ -601,27 +591,10 @@ void ContainmentInterface::mimeTypeRetrieved(KIO::Job *job, const QString &mimet
         return;
     } else {
 
-        QPoint posi; // will be overwritten with the event's position
-        if (m_dropPoints.contains(tjob)) {
-            posi = m_dropPoints.value(tjob);
-            qDebug() << "Received a suitable dropEvent at" << posi;
-        } else {
-            qDebug() << "Bailing out. Cannot find associated dropEvent related to the TransferJob";
-            clearDataForMimeJob(job);
-            return;
-        }
+        qDebug() << "Received a suitable dropEvent at " << m_dropMenu->dropPoint();
+        qDebug() << "Bailing out. Cannot find associated dropEvent related to the TransferJob";
 
-        QMenu *choices = m_dropMenus.value(tjob);
-        QList<QAction *> dropActions;
-        KIO::DropJob *dropJob = m_dropJobs.value(tjob);
-
-        if (!choices && !dropJob) {
-            qDebug() << "Bailing out. No QMenu or drop job found for this job.";
-            clearDataForMimeJob(job);
-            return;
-        }
-
-        qDebug() << "Creating menu for:" << mimetype  << posi;
+        qDebug() << "Creating menu for: " << mimetype;
 
         appletList << Plasma::PluginLoader::self()->listAppletMetaDataForMimeType(mimetype);
 
@@ -639,31 +612,25 @@ void ContainmentInterface::mimeTypeRetrieved(KIO::Job *job, const QString &mimet
 
         const bool isPlasmaPackage = (mimetype == QLatin1String("application/x-plasma"));
 
-        if (!appletList.isEmpty() || !wallpaperList.isEmpty() || isPlasmaPackage) {
+        if ((!appletList.isEmpty() || !wallpaperList.isEmpty() || isPlasmaPackage) && !m_dropMenu->isMultipleMimetypes()) {
             QAction *installPlasmaPackageAction = nullptr;
             if (isPlasmaPackage) {
-                if (choices) {
-                    choices->addSection(i18n("Plasma Package"));
-                    installPlasmaPackageAction = choices->addAction(QIcon::fromTheme(QStringLiteral("application-x-plasma")), i18n("Install"));
-                } else {
-                    QAction *action = new QAction(i18n("Plasma Package"), nullptr);
-                    action->setSeparator(true);
-                    dropActions << action;
+                QAction *action = new QAction(i18n("Plasma Package"), m_dropMenu);
+                action->setSeparator(true);
+                m_dropMenu->addAction(action);
+
+                installPlasmaPackageAction = new QAction(QIcon::fromTheme(QStringLiteral("application-x-plasma")), i18n("Install"), m_dropMenu);
+                m_dropMenu->addAction(installPlasmaPackageAction);
 
-                    installPlasmaPackageAction = new QAction(QIcon::fromTheme(QStringLiteral("application-x-plasma")), i18n("Install"), nullptr);
-                    Q_ASSERT(dropJob);
-                    dropActions << installPlasmaPackageAction;
-                    dropJob->setApplicationActions(dropActions);
-                }
                 const QString &packagePath = tjob->url().toLocalFile();
-                connect(installPlasmaPackageAction, &QAction::triggered, this, [this, posi, packagePath]() {
+                connect(installPlasmaPackageAction, &QAction::triggered, this, [this, packagePath]() {
                     using namespace KPackage;
                     PackageStructure *structure = PackageLoader::self()->loadPackageStructure(QStringLiteral("Plasma/Applet"));
                     Package package(structure);
 
                     KJob *installJob = package.update(packagePath);
-                    connect(installJob, &KJob::result, this, [this, packagePath, structure, posi](KJob *job) {
-                        auto fail = [job](const QString &text) {
+                    connect(installJob, &KJob::result, this, [this, packagePath, structure](KJob *job) {
+                        auto fail = [](const QString &text) {
                             KNotification::event(QStringLiteral("plasmoidInstallationFailed"), i18n("Package Installation Failed"),
                                                  text, QStringLiteral("dialog-error"), nullptr, KNotification::CloseOnTimeout, QStringLiteral("plasma_workspace"));
                         };
@@ -688,58 +655,45 @@ void ContainmentInterface::mimeTypeRetrieved(KIO::Job *job, const QString &mimet
                             return;
                         }
 
-                        createApplet(package.metadata().pluginId(), QVariantList(), QRect(posi, QSize(-1,-1)));
+                        createApplet(package.metadata().pluginId(), QVariantList(), QRect(m_dropMenu->dropPoint(), QSize(-1,-1)));
                     });
                 });
             }
 
-            if (choices) {
-                choices->addSection(i18n("Widgets"));
-            } else {
-                QAction *action = new QAction(i18n("Widgets"), nullptr);
-                action->setSeparator(true);
-                dropActions << action;
-            }
+            QAction *action = new QAction(i18n("Widgets"), m_dropMenu);
+            action->setSeparator(true);
+            m_dropMenu->addAction(action);
+
             foreach (const auto &info, appletList) {
                 const QString actionText = i18nc("Add widget", "Add %1", info.name());
-                QAction *action = new QAction(actionText, nullptr);
+                QAction *action = new QAction(actionText, m_dropMenu);
                 if (!info.iconName().isEmpty()) {
                     action->setIcon(QIcon::fromTheme(info.iconName()));
                 }
-                if (choices) {
-                    choices->addAction(action);
-                }
-                dropActions << action;
+                m_dropMenu->addAction(action);
                 action->setData(info.pluginId());
                 const QUrl url = tjob->url();
-                connect(action, &QAction::triggered, this, [this, action, posi, mimetype, url]() {
-                    Plasma::Applet *applet = createApplet(action->data().toString(), QVariantList(), QRect(posi, QSize(-1,-1)));
+                connect(action, &QAction::triggered, this, [this, action, mimetype, url]() {
+                    Plasma::Applet *applet = createApplet(action->data().toString(), QVariantList(), QRect(m_dropMenu->dropPoint(), QSize(-1,-1)));
                     setAppletArgs(applet, mimetype, url.toString());
                 });
             }
             {
-                QAction *action = new QAction(i18nc("Add icon widget", "Add Icon"), nullptr);
-                if (choices) {
-                    choices->addAction(action);
-                }
-                dropActions << action;
+                QAction *action = new QAction(i18nc("Add icon widget", "Add Icon"), m_dropMenu);
+                m_dropMenu->addAction(action);
                 action->setData(QStringLiteral("org.kde.plasma.icon"));
                 const QUrl url = tjob->url();
-                connect(action, &QAction::triggered, this, [this, action, posi, mimetype, url](){
-                    Plasma::Applet *applet = createApplet(action->data().toString(), QVariantList(), QRect(posi, QSize(-1,-1)));
+                connect(action, &QAction::triggered, this, [this, action, mimetype, url](){
+                    Plasma::Applet *applet = createApplet(action->data().toString(), QVariantList(), QRect(m_dropMenu->dropPoint(), QSize(-1,-1)));
                     setAppletArgs(applet, mimetype, url.toString());
                 });
             }
 
             QHash<QAction *, QString> actionsToWallpapers;
             if (!wallpaperList.isEmpty())  {
-                if (choices) {
-                    choices->addSection(i18n("Wallpaper"));
-                } else {
-                    QAction *action = new QAction(i18n("Wallpaper"), nullptr);
-                    action->setSeparator(true);
-                    dropActions << action;
-                }
+                QAction *action = new QAction(i18n("Wallpaper"), m_dropMenu);
+                action->setSeparator(true);
+                m_dropMenu->addAction(action);
 
                 QMap<QString, KPluginMetaData> sorted;
                 foreach (const auto &info, appletList) {
@@ -748,15 +702,11 @@ void ContainmentInterface::mimeTypeRetrieved(KIO::Job *job, const QString &mimet
 
                 foreach (const KPluginMetaData &info, wallpaperList) {
                     const QString actionText = i18nc("Set wallpaper", "Set %1", info.name());
-                    QAction *action = new QAction(actionText, nullptr);
+                    QAction *action = new QAction(actionText, m_dropMenu);
                     if (!info.iconName().isEmpty()) {
                         action->setIcon(QIcon::fromTheme(info.iconName()));
                     }
-
-                    if (choices) {
-                        choices->addAction(action);
-                    }
-                    dropActions << action;
+                    m_dropMenu->addAction(action);
                     actionsToWallpapers.insert(action, info.pluginId());
                     const QUrl url = tjob->url();
                     connect(action, &QAction::triggered, this, [this, url]() {
@@ -767,44 +717,27 @@ void ContainmentInterface::mimeTypeRetrieved(KIO::Job *job, const QString &mimet
                     });
                 }
             }
-
         } else {
             //case in which we created the menu ourselves, just the "fetching type entry, directly create the icon applet
-            if (choices) {
-                Plasma::Applet *applet = createApplet(QStringLiteral("org.kde.plasma.icon"), QVariantList(), QRect(posi, QSize(-1,-1)));
+            if (!m_dropMenu->isDropjobMenu()) {
+                Plasma::Applet *applet = createApplet(QStringLiteral("org.kde.plasma.icon"), QVariantList(), QRect(m_dropMenu->dropPoint(), QSize(-1,-1)));
                 setAppletArgs(applet, mimetype, tjob->url().toString());
             } else {
                 QAction *action;
-                if (choices) {
-                    choices->addSection(i18n("Widgets"));
-                    action = choices->addAction(i18nc("Add icon widget", "Add Icon"));
-                } else {
-                    QAction *sep = new QAction(i18n("Widgets"), nullptr);
-                    sep->setSeparator(true);
-                    dropActions << sep;
-                    // we can at least create an icon as a link to the URL
-                    action = new QAction(i18nc("Add icon widget", "Add Icon"), nullptr);
-                    dropActions << action;
-                }
+                QAction *sep = new QAction(i18n("Widgets"), m_dropMenu);
+                sep->setSeparator(true);
+                m_dropMenu->addAction(sep);
+                // we can at least create an icon as a link to the URL
+                action = new QAction(i18nc("Add icon widget", "Add Icon"), m_dropMenu);
+                m_dropMenu->addAction(action);
 
                 const QUrl url = tjob->url();
-                connect(action, &QAction::triggered, this, [this, posi, mimetype, url](){
-                    Plasma::Applet *applet = createApplet(QStringLiteral("org.kde.plasma.icon"), QVariantList(), QRect(posi, QSize(-1,-1)));
+                connect(action, &QAction::triggered, this, [this, mimetype, url](){
+                    Plasma::Applet *applet = createApplet(QStringLiteral("org.kde.plasma.icon"), QVariantList(), QRect(m_dropMenu->dropPoint(), QSize(-1,-1)));
                     setAppletArgs(applet, mimetype, url.toString());
                 });
             }
         }
-
-        if (choices) {
-            // HACK If the QMenu becomes empty at any point after the "determining mimetype"
-            // popup was shown, it self-destructs, does not matter if we call clear() or remove
-            // the action manually, hence we remove the aforementioned item after we populated the menu
-            choices->removeAction(choices->actions().at(0));
-            choices->exec();
-        } else {
-            dropJob->setApplicationActions(dropActions);
-        }
-
         clearDataForMimeJob(tjob);
     }
 }
@@ -859,25 +792,24 @@ void ContainmentInterface::loadWallpaper()
         return;
     }
 
-    if (!m_containment->wallpaper().isEmpty()) {
-        delete m_wallpaperInterface;
-
+    if (!m_wallpaperInterface && !m_containment->wallpaper().isEmpty()) {
         m_wallpaperInterface = new WallpaperInterface(this);
+
         m_wallpaperInterface->setZ(-1000);
         //Qml seems happier if the parent gets set in this way
         m_wallpaperInterface->setProperty("parent", QVariant::fromValue(this));
 
+        connect(m_wallpaperInterface, &WallpaperInterface::isLoadingChanged, this, &AppletInterface::updateUiReadyConstraint);
+
         //set anchors
         QQmlExpression expr(qmlObject()->engine()->rootContext(), m_wallpaperInterface, QStringLiteral("parent"));
         QQmlProperty prop(m_wallpaperInterface, QStringLiteral("anchors.fill"));
         prop.write(expr.evaluate());
 
         m_containment->setProperty("wallpaperGraphicsObject", QVariant::fromValue(m_wallpaperInterface));
-    } else {
-        if (m_wallpaperInterface) {
-            m_wallpaperInterface->deleteLater();
-            m_wallpaperInterface = nullptr;
-        }
+    } else if (m_wallpaperInterface && m_containment->wallpaper().isEmpty()) {
+        m_wallpaperInterface->deleteLater();
+        m_wallpaperInterface = nullptr;
     }
 }
 
@@ -919,7 +851,7 @@ QList<QObject *> ContainmentInterface::actions() const
 
     i = 0;
     foreach (QAction *a, m_containment->corona()->actions()->actions()) {
-        if (a->objectName() == QStringLiteral("lock widgets") || a->menu()) {
+        if (a->objectName() == QLatin1String("lock widgets") || a->menu()) {
             //It is up to the Containment to decide if the user is allowed or not
             //to lock/unluck the widgets, so corona should not add one when there is none
             //(user is not allow) and it shouldn't add another one when there is already
@@ -987,7 +919,7 @@ void ContainmentInterface::mousePressEvent(QMouseEvent *event)
     Plasma::Applet *applet = nullptr;
     foreach (QObject *appletObject, m_appletInterfaces) {
         if (AppletInterface *ai = qobject_cast<AppletInterface *>(appletObject)) {
-            if (ai->isVisible() && ai->contains(ai->mapFromItem(this, event->posF()))) {
+            if (ai->isVisible() && ai->contains(ai->mapFromItem(this, event->localPos()))) {
                 applet = ai->applet();
                 break;
             } else {
@@ -1086,10 +1018,10 @@ void ContainmentInterface::wheelEvent(QWheelEvent *event)
         return;
     }
 
-    m_wheelDelta += event->delta();
+    m_wheelDelta += event->angleDelta().y();
 
     // Angle delta 120 for common "one click"
-    // See: http://qt-project.org/doc/qt-5/qml-qtquick-wheelevent.html#angleDelta-prop
+    // See: https://doc.qt.io/qt-5/qml-qtquick-wheelevent.html#angleDelta-prop
     while (m_wheelDelta >= 120) {
         m_wheelDelta -= 120;
         plugin->performPreviousAction();
@@ -1135,34 +1067,14 @@ void ContainmentInterface::addAppletActions(QMenu *desktopMenu, Plasma::Applet *
         }
     }
 
-    QMenu *containmentMenu = new QMenu(i18nc("%1 is the name of the containment", "%1 Options", m_containment->title()), desktopMenu);
-    if (m_containment->containmentType() != Plasma::Types::DesktopContainment) {
-        addContainmentActions(containmentMenu, event);
-    }
-
-    if (!containmentMenu->isEmpty()) {
-        int enabled = 0;
-        //count number of real actions
-        QListIterator<QAction *> actionsIt(containmentMenu->actions());
-        while (enabled < 3 && actionsIt.hasNext()) {
-            QAction *action = actionsIt.next();
-            if (action->isVisible() && !action->isSeparator()) {
-                ++enabled;
-            }
-        }
-
-        if (enabled) {
-            //if there is only one, don't create a submenu
-            if (enabled < 2) {
-                foreach (QAction *action, containmentMenu->actions()) {
-                    if (action->isVisible() && !action->isSeparator()) {
-                        desktopMenu->addAction(action);
-                    }
-                }
-            } else {
-                desktopMenu->addMenu(containmentMenu);
-            }
+    desktopMenu->addSeparator();
+    if (m_containment->containmentType() == Plasma::Types::DesktopContainment) {
+        auto action = m_containment->corona()->actions()->action(QStringLiteral("edit mode"));
+        if (action) {
+            desktopMenu->addAction(action);
         }
+    } else {
+        addContainmentActions(desktopMenu, event);
     }
 
     if (m_containment->immutability() == Plasma::Types::Mutable &&
@@ -1183,7 +1095,7 @@ void ContainmentInterface::addAppletActions(QMenu *desktopMenu, Plasma::Applet *
 void ContainmentInterface::addContainmentActions(QMenu *desktopMenu, QEvent *event)
 {
     if (m_containment->corona()->immutability() != Plasma::Types::Mutable &&
-            !KAuthorized::authorizeKAction(QStringLiteral("plasma/containment_actions"))) {
+            !KAuthorized::authorizeAction(QStringLiteral("plasma/containment_actions"))) {
         //qDebug() << "immutability";
         return;
     }
@@ -1223,4 +1135,14 @@ void ContainmentInterface::addContainmentActions(QMenu *desktopMenu, QEvent *eve
     return;
 }
 
+bool ContainmentInterface::isLoading() const
+{
+    bool loading = AppletInterface::isLoading();
+    if (m_wallpaperInterface) {
+        loading |= m_wallpaperInterface->isLoading();
+    }
+    return loading;
+}
+
+
 #include "moc_containmentinterface.cpp"
