@@ -28,6 +28,7 @@
 #include "appletinterface.h"
 
 class WallpaperInterface;
+class DropMenu;
 
 namespace KIO
 {
@@ -54,6 +55,7 @@ class ContainmentInterface : public AppletInterface
 
     /**
      * List of applets this containment has: the containments
+     * KF6: this should be AppletQuickItem *
      */
     Q_PROPERTY(QList <QObject *> applets READ applets NOTIFY appletsChanged)
 
@@ -78,10 +80,11 @@ class ContainmentInterface : public AppletInterface
     Q_PROPERTY(QList<QObject *> actions READ actions NOTIFY actionsChanged)
 
     /**
-     * True when the containment is in an edit mode that allows to move
+     * True when the Plasma Shell is in an edit mode that allows to move
      * things around: it's different from userConfiguring as it's about
      * editing plasmoids inside the containment, rather than the containment
-     * settings dialog itself
+     * settings dialog itself.
+     * This is global for the whole Plasma process, all containments will have the same value for editMode
      */
     Q_PROPERTY(bool editMode READ isEditMode WRITE setEditMode NOTIFY editModeChanged)
 
@@ -149,6 +152,11 @@ public:
      */
     Q_INVOKABLE QPointF adjustToAvailableScreenRegion(int x, int y, int w, int h) const;
 
+    /**
+     * @returns a named action from global Corona's actions
+     */
+    Q_INVOKABLE QAction *globalAction(QString name) const;
+
     bool isEditMode() const;
     void setEditMode(bool edit);
 
@@ -158,15 +166,17 @@ public:
     }
 
 protected:
-    void init() Q_DECL_OVERRIDE;
-    void mousePressEvent(QMouseEvent *event) Q_DECL_OVERRIDE;
-    void mouseReleaseEvent(QMouseEvent *event) Q_DECL_OVERRIDE;
-    void wheelEvent(QWheelEvent *event) Q_DECL_OVERRIDE;
-    void keyPressEvent(QKeyEvent *event) Q_DECL_OVERRIDE;
+    void init() override;
+    void mousePressEvent(QMouseEvent *event) override;
+    void mouseReleaseEvent(QMouseEvent *event) override;
+    void wheelEvent(QWheelEvent *event) override;
+    void keyPressEvent(QKeyEvent *event) override;
 
     void addAppletActions(QMenu *desktopMenu, Plasma::Applet *applet, QEvent *event);
     void addContainmentActions(QMenu *desktopMenu, QEvent *event);
 
+    virtual bool isLoading() const override;
+
 Q_SIGNALS:
     /**
      * Emitted when an applet is added
@@ -209,15 +219,11 @@ private:
 
     WallpaperInterface *m_wallpaperInterface;
     QList<QObject *> m_appletInterfaces;
-    QHash<KJob *, QPoint> m_dropPoints;
-    QHash<KJob *, QMenu *> m_dropMenus;
-    QHash<KJob *, QJSValue *> m_dropCallbacks;
-    QHash<KJob *, KIO::DropJob *> m_dropJobs;
     KActivities::Info *m_activityInfo;
     QPointer<Plasma::Containment> m_containment;
-    QWeakPointer<QMenu> m_contextMenu;
+    QPointer<QMenu> m_contextMenu;
+    QPointer<DropMenu> m_dropMenu;
     int m_wheelDelta;
-    bool m_editMode : 1;
     friend class AppletInterface;
 };
 
